import configparser
from math import *
from matplotlib.pyplot import *
from numpy import *


def load_conf(file, section):
    conf = configparser.ConfigParser()
    conf.read(file, 'utf-8')
    return conf.items(section)


def calculate_coef(p0, p1, p2, p3):
    c = 3*(p1 - p0)
    b = 3*(p2 - p1) - c
    a = p3 - p0 - c - b
    return c, b, a


def Bezier(plist, t):
    p0, p1, p2, p3 = plist
    c, b, a = calculate_coef(p0, p1, p2, p3)
    tsquared = t**2
    tcubic = tsquared*t
    return a*tcubic + b*tsquared + c*t + p0

GAP_RATIO = 4
radius_a, radius_b = .33, .335
sm_radius=(radius_b-radius_a)/2

blocklength = 0.01  # gene块的长度
blockthick = 0.007
shiftratio = 1.1


def get_species_and_chr(spelist):
    iscompletegenome = {}
    assert (spelist),'Please enter species name!!!'
    specieslist = list(set([ s[0:3] for s in spelist]))
    num = [1 if s==s[0:3] else 0 for s in specieslist]
    for i,v in zip(spelist,num):
        iscompletegenome[i[0:3]]=v
    # print(iscompletegenome)

    return specieslist,iscompletegenome


def get_chr_lenth(spelist,fpchrolen,iscompletegenome):
    chro2len = {}
    labels = []
    flag = False
    for row in fpchrolen:
        chro, length = row.split('\t')
        # chro = chro.upper()
        if len(chro) > 5:
            continue
        if iscompletegenome[chro[0:3]] == 1:
            flag = True
        else:
            if chro in spelist:
                flag = True
        if flag is True:
            chro2len[chro] = int(length)
            labels.append(chro)
    fpchrolen.close()
    return chro2len,labels


def input_gff(fpgff):
    gene2pos={}
    gene2chain = {}
    gene_labels = {}
    for row in fpgff:
        if not row.strip():
            continue
        ch, gene, start, end = row.split()
        start = int(start)
        end = int(end)
        gene2pos[gene] = int(start)
        gene2chain[gene] = int((end-start)/abs(end -start))
        # gene_labels[gene] = ch.upper()
        gene_labels[gene] = ch
    fpgff.close()
    return gene2pos,gene2chain,gene_labels


def input_ka_ks(fpgenefamilyinf,genelabels):
    genepair2Ks = {}
    genes = []

    for row in fpgenefamilyinf:
        gene1, gene2, Ka, Ks = row.split('\t')
        if gene1 < gene2:
            genepair = gene1 + " " + gene2
        else:
            genepair = gene2 + " " + gene1
        if len(genelabels[gene1]) < 6  and len(genelabels[gene2]) < 6 :  # 根据染色体改比如  Oeu01
            genepair2Ks[genepair] = float(Ks)
        if gene1 not in genes:
            if len(genelabels[gene1]) < 6:
                genes.append(gene1)
        if gene2 not in genes:
            if len(genelabels[gene2]) < 6:
                genes.append(gene2)
    fpgenefamilyinf.close()

    return genes,genepair2Ks


def rad_to_coord(angle, radius):
    return radius*cos(angle), radius*sin(angle)


def to_deg(bp, total):
    return bp*360./total


def to_radian(bp, total):
    return radians(bp*360./total)


def plot_arc(start, stop, radius):
    t = arange(start, stop, pi/720.)
    x, y = radius*cos(t), radius*sin(t)
    plot(x, y, "k-", alpha=.5)


def plot_cap(angle, clockwise=True, radius=sm_radius):
    if clockwise: t = arange(angle, angle+pi, pi/30.)
    else: t = arange(angle, angle-pi, -pi/30.)
    x, y = radius*cos(t), radius*sin(t)
    middle_r = (radius_a+radius_b)/2
    x, y = x + middle_r*cos(angle), y + middle_r*sin(angle)
    plot(x, y, "k-", alpha=.5)


def plot_arc_block(start,radius):
    t = arange(start, start+blocklength, pi/720.)
    x,y = radius * cos(t), radius*sin(t)
    plot(x, y, "b-", alpha=1)


def plot_arc_block1(start,radius,col):
    t = arange(start, start+blocklength, pi/720.)
    x,y = radius * cos(t), radius*sin(t)
    # print(x[:2], y[:2],)
    plot(x, y, "-", alpha=1,color=col, lw=4)  # lw=4 线宽 gene块的厚度

def plot_arc_block2(start,radius,col):
    t = arange(start, start+blocklength, pi/720.)
    x,y = radius * cos(t), radius*sin(t)
    # print(x[:2], y[:2],)
    plot(x, y, "-", alpha=1,color=col, lw=1.5)  # lw=4 线宽 gene块的厚度


def sort_gene(genes,gene2pos,start_list,labels,genelabels):
    genessorted = []
    geneno = 0
    for i in range(len(genes)):
        if geneno == 0:
            genessorted.append(genes[0])
            geneno = geneno + 1
        else:
            firstgene = genessorted[0]
            lastgene = genessorted[-1]
            chf = genelabels[firstgene]
            chl = genelabels[lastgene]
            posf = gene2pos[firstgene] + start_list[labels.index(chf)]
            posl = gene2pos[lastgene] + start_list[labels.index(chl)]
            chi = genelabels[genes[i]]
            posi = gene2pos[genes[i]] + start_list[labels.index(chi)]
            if posi <= posf:
                genessorted[0:0] = [genes[i]]
            elif posi >= posl:
                genessorted.append(genes[i])
            else:
                for j in range(len(genessorted) - 1):
                    chj = genelabels[genessorted[j]]
                    posj = gene2pos[genessorted[j]] + start_list[labels.index(chj)]
                    chj1 = genelabels[genessorted[j + 1]]
                    posj1 = gene2pos[genessorted[j + 1]] + start_list[labels.index(chj1)]
                    if posj < posi < posj1:
                        genessorted[j + 1:j + 1] = [genes[i]]
    return genessorted
