import os
import click

import famcicle
from famcicle.inner import run_inner
from famcicle.excircle import run_excircle
from famcicle.circos import run_circos
from famcicle.func_ import load_conf

path = famcicle.__path__[0]

@click.group()
@click.version_option(version='0.1',message='version %(version)s')
def main():
    '''
    This is a software to draw Tandem repeat gene diagram,Contains three drawing subcommands

    \b
    If the number of genes is less than 100, then inner is selected,
    Select Excircle if greater than 100
    To draw a circle graph from a different pair hmm_model, select circos
    '''


@main.command('inner', help='Draw a circle diagram inside a circle')
@click.option('-c', '--conf', help='Enter configuration hmm_model',type=None)
@click.option('-e', '--example', help='Displays the contents of the configuration hmm_model',
              is_flag=True)
def inner(conf,example):
    try:
        if example:
            return click.echo(open(os.path.join(path, 'conf/inner.conf')).read()+'\n')
        if conf:
            if os.path.exists(conf) and os.path.isfile(conf):
                options = load_conf(conf, 'inner')
                run_inner(options)
            else:
                click.echo('Please enter the correct hmm_model')
    except PermissionError:
        click.echo('Permission denied')


@main.command('excircle', help='Draw a diagram of the emission circle')
@click.option('-c', '--conf', help='Enter configuration hmm_model')
@click.option('-e', '--example', help='Displays the contents of the configuration hmm_model',
              is_flag=True)
def excircle(conf,example):
    try:
        if example:
            return click.echo(open(os.path.join(path, 'conf/excircle.conf')).read()+'\n')
        if conf:
            if os.path.exists(conf) and os.path.isfile(conf):
                options = load_conf(conf, 'excircle')
                run_excircle(options)
            else:
                click.echo('Please enter the correct hmm_model')
    except PermissionError:
        click.echo('Permission denied')


@main.command('circos', help='Draw a pair circle')
@click.option('-c', '--conf', help='Enter configuration hmm_model,'
                                 'First, you need to select Format parameter')
@click.option('-e', '--example', help='Displays the contents of the configuration hmm_model',
              is_flag=True)
@click.option('--format',type=click.Choice(['processed', 'blast','mcscan','colinearscan'],
                                                case_sensitive=False),
              default='processed',show_default=True,
              help='''Pair hmm_model format:\n
                   processed:id1\\tid2 or id1\\tid2\\tcolor\n
                   blast:BLAST results raw hmm_model\n
                   mcscan:MCScan results to the raw hmm_model\n
                   colinearscan:ColinearScan results to the raw hmm_model
                   ''')
def circle(conf,example,format):
    try:
        if example:
            return click.echo(open(os.path.join(path, 'conf/circos.conf')).read()+'\n')
        if conf:
            if os.path.exists(conf) and os.path.isfile(conf):
                options = load_conf(conf, 'circos')
                circos = run_circos(options,format)
                circos.run()
            else:
                click.echo('Please enter the correct hmm_model')
    except PermissionError:
        click.echo('Permission denied')